@@ -1,60 +1,19 @@
-package org.apache.commons.net.telnet;
-
-/* ====================================================================
+/*
+ * Copyright 2003-2004 The Apache Software Foundation
  *
- * The Apache Software License, Version 1.1
+ * Licensed under the Apache License, Version 2.0 (the "License");
+ * you may not use this file except in compliance with the License.
+ * You may obtain a copy of the License at
  *
- * Copyright (c) 2003 The Apache Software Foundation.  All rights
- * reserved.
+ *     http://www.apache.org/licenses/LICENSE-2.0
  *
- * Redistribution and use in source and binary forms, with or without
- * modification, are permitted provided that the following conditions
- * are met:
- *
- * 1. Redistributions of source code must retain the above copyright
- *    notice, this list of conditions and the following disclaimer.
- *
- * 2. Redistributions in binary form must reproduce the above copyright
- *    notice, this list of conditions and the following disclaimer in
- *    the documentation and/or other materials provided with the
- *    distribution.
- *
- * 3. The end-user documentation included with the redistribution, if
- *    any, must include the following acknowledgement:
- *       "This product includes software developed by the
- *        Apache Software Foundation (http://www.apache.org/)."
- *    Alternately, this acknowledgement may appear in the software itself,
- *    if and wherever such third-party acknowledgements normally appear.
- *
- * 4. The names "The Jakarta Project", "Tomcat", and "Apache Software
- *    Foundation" must not be used to endorse or promote products derived
- *    from this software without prior written permission. For written
- *    permission, please contact apache@apache.org.
- *
- * 5. Products derived from this software may not be called "Apache"
- *    nor may "Apache" appear in their names without prior written
- *    permission of the Apache Software Foundation.
- *
- * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
- * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
- * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
- * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
- * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
- * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
- * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
- * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
- * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
- * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
- * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
- * SUCH DAMAGE.
- * ====================================================================
- *
- * This software consists of voluntary contributions made by many
- * individuals on behalf of the Apache Software Foundation.  For more
- * information on the Apache Software Foundation, please see
- * <http://www.apache.org/>.
+ * Unless required by applicable law or agreed to in writing, software
+ * distributed under the License is distributed on an "AS IS" BASIS,
+ * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+ * See the License for the specific language governing permissions and
+ * limitations under the License.
  */
-
+package org.apache.commons.net.telnet;
 import junit.framework.TestCase;
 import java.io.InputStream;
 import java.io.OutputStream;
@@ -114,6 +73,23 @@ public class TelnetClientTest extends TestCase implements TelnetNotificationHand
         tc3.connect("127.0.0.1", 3335);
         Thread.sleep(1000);
     }
+    /***
+     * open connections (ReaderThread test).
+     ***/
+    protected void openConnReaderThread() throws Exception
+    {
+        server1 = new TelnetTestSimpleServer(3333);
+        server2 = new TelnetTestSimpleServer(3334);
+
+        tc1 = new TelnetClient();
+        tc1.setReaderThread(false);
+        tc2 = new TelnetClient();
+        tc2.setReaderThread(true);
+
+        tc1.connect("127.0.0.1", 3333);
+        tc2.connect("127.0.0.1", 3334);
+        Thread.sleep(1000);
+    }
 
     /***
      * tests the initial condition of the sessions
@@ -448,7 +424,7 @@ public class TelnetClientTest extends TestCase implements TelnetNotificationHand
         };
 
         byte buffread2b[] = new byte[11];
-    
+
         openConnections();
 
         numdo = 0;
@@ -673,6 +649,79 @@ public class TelnetClientTest extends TestCase implements TelnetNotificationHand
         assertTrue(stopspy_ok);
     }
 
+    /***
+     * test of setReaderThread
+     ***/
+    public void testSetReaderThread() throws Exception
+    {
+        boolean negotiation1_ok = false;
+        boolean negotiation2_ok = false;
+        boolean read_ok = false;
+        byte buffread1[] = new byte[6];
+        byte send1[] =
+        {
+            (byte)TelnetCommand.IAC, (byte)TelnetCommand.DO, (byte)15,
+            (byte)TelnetCommand.IAC, (byte)TelnetCommand.WILL, (byte)15,
+        };
+        byte expected1[] =
+        {
+            (byte)TelnetCommand.IAC, (byte)TelnetCommand.WONT, (byte)15,
+            (byte)TelnetCommand.IAC, (byte)TelnetCommand.DONT, (byte)15,
+        };
+
+
+        openConnReaderThread();
+
+        InputStream is1 = server1.getInputStream();
+        OutputStream os1 = server1.getOutputStream();
+        is1.skip(is1.available());
+        os1.write(send1);
+        os1.flush();
+        os1.write("A".getBytes());
+        os1.flush();
+        Thread.sleep(1000);
+        InputStream instr = tc1.getInputStream();
+        byte[] buff = new byte[4];
+        int ret_read = 0;
+
+        ret_read = instr.read(buff);
+        if((ret_read == 1) && (buff[0] == 'A'))
+        {
+            read_ok = true;
+        }
+
+        if(is1.available() == 6)
+        {
+            is1.read(buffread1);
+
+            if(equalBytes(buffread1, expected1))
+                negotiation1_ok = true;
+        }
+
+        InputStream is2 = server2.getInputStream();
+        OutputStream os2 = server2.getOutputStream();
+        Thread.sleep(1000);
+        is2.skip(is2.available());
+        os2.write(send1);
+        os2.flush();
+        Thread.sleep(1000);
+        if(is2.available() == 6)
+        {
+            is2.read(buffread1);
+
+            if(equalBytes(buffread1, expected1))
+                negotiation2_ok = true;
+        }
+
+        closeConnReaderThread();
+
+        assertTrue(!tc1.getReaderThread());
+        assertTrue(tc2.getReaderThread());
+        assertTrue(read_ok);
+        assertTrue(negotiation1_ok);
+        assertTrue(negotiation2_ok);
+    }
+
     /***
      * closes all the connections
      ***/
@@ -695,6 +744,25 @@ public class TelnetClientTest extends TestCase implements TelnetNotificationHand
         {
         }
     }
+    /***
+     * closes all the connections (ReaderThread test)
+     ***/
+    protected void closeConnReaderThread()
+    {
+        try
+        {
+            server1.disconnect();
+            server1.stop();
+            tc1.disconnect();
+            server2.disconnect();
+            server2.stop();
+            tc2.disconnect();
+            Thread.sleep(1000);
+        }
+        catch (Exception e)
+        {
+        }
+    }
 
     /***
      * Helper method. compares two arrays of int
